from Components.ActionMap import ActionMap, NumberActionMap
from Components.config import config, ConfigSubsection, ConfigOnOff, ConfigYesNo, ConfigSelection, ConfigSelectionNumber, getConfigListEntry
from Components.ConfigList import ConfigListScreen
from Components.HdmiCec import hdmi_cec
from Components.Label import Label
from Components.Sources.StaticText import StaticText
from Components.VolumeControl import VolumeControl
from Plugins.Plugin import PluginDescriptor
from Screens.Menu import MainMenu
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from enigma import  getDesktop, eCec, eDVBVolumecontrol, eTimer, quitMainloop
from . import _
from time import strftime as t_strftime, localtime as t_localtime

def getVideoModes():
	with open('/proc/stb/video/videomode_choices') as vFile:
		return [(x, x) for x in vFile.readline().split()]

config.plugins.merlinhdmiin = ConfigSubsection()
config.plugins.merlinhdmiin.sourceresolution = ConfigSelection(default="720p", choices=getVideoModes() )
config.plugins.merlinhdmiin.bypassedid = ConfigOnOff(default=False)
config.plugins.merlinhdmiin.showinmainmenu = ConfigYesNo(default=True)
config.plugins.merlinhdmiin.adjustvolume = ConfigYesNo(default=False)
config.plugins.merlinhdmiin.volume = ConfigSelectionNumber(5, 100, 5, default = 70)
config.plugins.merlinhdmiin.showvolumechange = ConfigYesNo(default=True)

def Plugins(**kwargs):
	return [
		PluginDescriptor(name=_("Merlin HDMI-In"), description=_("Passthrough HDMI-In to HDMI-Out"), where = PluginDescriptor.WHERE_EXTENSIONSMENU, icon="plugin.png", fnc=openMerlinHdmiIn),
		PluginDescriptor(where = [PluginDescriptor.WHERE_SESSIONSTART, PluginDescriptor.WHERE_AUTOSTART], fnc = autostart ),
		PluginDescriptor(where = PluginDescriptor.WHERE_MENU, description=_("Passthrough HDMI-In to HDMI-Out"), fnc = menu ),
		PluginDescriptor(where = PluginDescriptor.WHERE_MENU, description=_("Configure Merlin HDMI-IN"),  fnc = setup ),
		]

def openMerlinHdmiIn(session, **kwargs):
	session.openWithCallback(closeMenu, MerlinHdmiIn)
	
def closeMenu(session):
	if isinstance (session.current_dialog, MainMenu):
		session.current_dialog.closeRecursive()		

class MerlinCec(object):

	DEVICE_TYPE = {
		eCec.ADDR_TV : _("TV"),
		eCec.ADDR_RECORDING_DEVICE_1 : _("Recording Device"),
		eCec.ADDR_RECORDING_DEVICE_2 : _("Recording Device 2"),
		eCec.ADDR_TUNER_1 : _("Tuner"),
		eCec.ADDR_PLAYBACK_DEVICE_1 : _("Playback Device"),
		eCec.ADDR_AUDIO_SYSTEM : _("Audio System"),
		eCec.ADDR_TUNER_2 : _("Tuner 2"),
		eCec.ADDR_TUNER_3 : _("Tuner 3"),
		eCec.ADDR_PLAYBACK_DEVICE_2 : _("Playback Device 2"),
		eCec.ADDR_RECORDING_DEVICE_3 : _("Recording Device 3"),
		eCec.ADDR_TUNER_4 : _("Tuner 4"),
		eCec.ADDR_PLAYBACK_DEVICE_3 : _("Playback Device 3"),
	}

	def __init__(self):
		self.session = None
		self.cecIsNowActive_conn = None
		self.cecIsReady_conn = None
		self.cecMsg_conn = None
		self._started = False
		
	def start(self, session):
		if self._started:
			return

		self.cecIsNowActive_conn = hdmi_cec.instance.isNowActive.connect(self.__sourceActive)
		self.cecIsReady_conn = hdmi_cec.instance.ready.connect(self.__isReady)
		self.cecMsg_conn = hdmi_cec.instance.receivedMessage.connect(self.__gotMessage)
		self.cecRawMsg_conn = hdmi_cec.instance.receivedRaw.connect(self.__gotMessage)
		
		self._started = True		

	def checkActiveSource(self):
		return hdmi_cec.instance.isActiveSource()

	def getDevice(self):
		deviceList = hdmi_cec.instance.getKnownDevices()
		print "devList", deviceList
		print "len", len(deviceList)
		devices = []
		for device in deviceList:
			deviceType = self.DEVICE_TYPE.get(device.logicalAddress(), _("Other"))
			text = "{0} - {1} {2}".format(deviceType, device.readableVendor(), device.deviceName())
			print "text", text
			devices.append(text)	
			print "device", device
			print "deviceType", deviceType
			print "logAddr", device.logicalAddress()
			print "vendor", device.readableVendor()
			print "devName", device.deviceName()

	def __gotMessage(self, *args):
		print "[MerlinHDMI] - HDMI_IN got message", args
	
	def __sourceActive(self, *args):
		print "[MerlinHDMI] - HDMI_IN active"
		
	def __isReady(self, *args):
		print "[MerlinHDMI] - HDMI_IN ready"
			
merlinCec = MerlinCec()		

def autostart(reason, **kwargs):
	if reason == 0:
		session = kwargs.get('session', None)
		if session is not None:
			merlinCec.start(session)

def menu(menuid, **kwargs):
	if menuid == 'mainmenu':
		if config.plugins.merlinhdmiin.showinmainmenu.value:
			return [(_("Merlin HDMI-In"), openMerlinHdmiIn, "merlin_hdmi_in", None)]
	return []
			
def setup(menuid, **kwargs):
	if menuid == 'merlin':
		return [(_("Merlin HDMI-In Setup"), openSetup , "merlin_hdmi_in_setup", None)]
	return []
	
def openSetup(session):
	session.open(MerlinHdmiInSetup)

class MerlinHdmiIn(Screen):
	skin = """
		<screen position="0,0" size="%d,%d" flags="wfNoBorder" zPosition="100" title="Merlin HDMI-Input" transparent="1">
		</screen>
	""" %(getDesktop(0).size().width(), getDesktop(0).size().height())

	def __init__(self, session, args=0):
		self.session = session
		Screen.__init__(self, session)
		
		self["OkCancelActions"] = ActionMap(["OkCancelActions"],
		{
			"ok":			merlinCec.getDevice,
			"cancel":		self.exitPlugin,
		}, -1)
		
		self.backUpDict = {}		
		self.currentService = None
		self.volctrl = None
		self.volumeControlInstance = None
		self.oldVolume = None
		self.adjustVolumeActive = config.plugins.merlinhdmiin.adjustvolume.value
		self.showVolume = config.plugins.merlinhdmiin.showvolumechange.value
		self.startHdmiIn()

	def createSummary(self):
		return MerlinHdmiInScreenSummary
		
	def startHdmiIn(self):
		self.currentService = self.session.nav.getCurrentlyPlayingServiceReference()
		if self.adjustVolumeActive:
			self.volctrl = eDVBVolumecontrol.getInstance()
			# save volume value
			self.oldVolume = self.volctrl.getVolume()
		self.session.nav.stopService()
		if self.adjustVolumeActive:
			if self.showVolume:
				self.volumeControlInstance = VolumeControl.instance
			# adjust volume after service is stopped to not interfere AutomaticVolumeAdjustment
			self.adjustVolume(config.plugins.merlinhdmiin.volume.getValue())
		self.writeData('/proc/stb/video/videomode', config.plugins.merlinhdmiin.sourceresolution.value)
		self.writeData('/proc/stb/video/videomode_50hz')
		self.writeData('/proc/stb/video/videomode_60hz')
		self.writeData('/proc/stb/audio/hdmi_rx_monitor', 'on', False)
		self.writeData('/proc/stb/hdmi-rx/0/hdmi_rx_monitor', 'on', False)
		#self.writeData('/proc/stb/audio/force_hdmi_audio_disable', '1')
		if config.plugins.merlinhdmiin.bypassedid.value:
			self.writeData('/proc/stb/hdmi/bypass_edid_checking', '00000001')
		else:
			self.writeData('/proc/stb/hdmi/bypass_edid_checking', '00000000')
		# does not really work
		merlinCec.getDevice()
	
	def writeData(self, file, value=None, backUp=True):
		with open(file, 'r+') as iWriteFile:
			if backUp:
				self.backUpDict[file] = iWriteFile.read().strip()
			if value is not None:
				iWriteFile.write(value)
			elif value is None and not backUp:
				iWriteFile.write(self.backUpDict.get(file))

	def exitPlugin(self):
		self.writeData('/proc/stb/audio/hdmi_rx_monitor', 'off', False)
		self.writeData('/proc/stb/hdmi-rx/0/hdmi_rx_monitor', 'off', False)
		if config.plugins.merlinhdmiin.bypassedid.value:
			self.writeData('/proc/stb/hdmi/bypass_edid_checking', '00000000', False)
		# the following 2 lines are only required to have a smooth transition to live tv when exiting
		self.writeData('/proc/stb/avs/0/input', 'aux', False)
		self.writeData('/proc/stb/avs/0/input', 'encoder', False)
		self.writeData('/proc/stb/video/videomode', None, False)
		self.writeData('/proc/stb/video/videomode_50hz', None, False)
		self.writeData('/proc/stb/video/videomode_60hz', None, False)
		# reset volume
		if self.adjustVolumeActive:
			self.adjustVolume(self.oldVolume)
			self.volumeControlInstance = None
			self.volctrl = None
		self.session.nav.playService(self.currentService)
		self.close(self.session)
		
	def adjustVolume(self, value):
		if self.volctrl is not None:
			self.volctrl.setVolume(value, value)
		if self.volumeControlInstance is not None and self.showVolume:
			self.volumeControlInstance.volumeDialog.setValue(value)
			self.volumeControlInstance.volumeDialog.show()
			self.volumeControlInstance.hideVolTimer.start(3000, True)		

class MerlinHdmiInSetup(ConfigListScreen, Screen):
	skin = """
		<screen position="center,center" size="700,350" title="Merlin HDMI-In Setup" >
			<widget name="config" position="10,10" size="680,250" scrollbarMode="showOnDemand" />
			<ePixmap pixmap="skin_default/buttons/button_red.png" position="10,300" size="25,25" />
       	 	<ePixmap pixmap="skin_default/buttons/button_green.png" position="320,300" size="25,25" />
       	 	<widget font="Regular; 28" foregroundColor="button" halign="left" position="45,293" render="Label" size="260,40" source="key_red" transparent="1" valign="center" zPosition="1" />
       	 	<widget font="Regular; 28" foregroundColor="button" halign="left" position="350,293" render="Label" size="260,40" source="key_green" transparent="1" valign="center" zPosition="1" />
		</screen>
	"""
	
	def __init__(self, session):
		Screen.__init__(self, session)

		self.list = []

		ConfigListScreen.__init__(self, self.list, session=session)

		self["key_red"] = StaticText(_("Cancel"))
		self["key_green"] = StaticText(_("OK"))

		self["actions"] = NumberActionMap(["SetupActions"], 
			{
				"cancel": self.keyCancel,
				"save": self.keySave,
			}, -2)
		
		self._changedEntry()
		self.onLayoutFinish.append(self.createList)
		
	def createList(self):
		self.list = []
		
		self.list.append(getConfigListEntry(_("Show in mainmenu"), config.plugins.merlinhdmiin.showinmainmenu))
		self.list.append(getConfigListEntry(_("Source resolution"), config.plugins.merlinhdmiin.sourceresolution))
		self.list.append(getConfigListEntry(_("Bypass EDID check"), config.plugins.merlinhdmiin.bypassedid))
		self.list.append(getConfigListEntry(_("Adjust volume for HDMI-IN"), config.plugins.merlinhdmiin.adjustvolume))
		if config.plugins.merlinhdmiin.adjustvolume.value:
			self.list.append(getConfigListEntry(_("Volume"), config.plugins.merlinhdmiin.volume))
			self.list.append(getConfigListEntry(_("Show volume indicator"), config.plugins.merlinhdmiin.showvolumechange))
		self["config"].setList(self.list)
		
	def keyLeft(self):
		ConfigListScreen.keyLeft(self)
		self.updateConfigOptions()
		
	def keyRight(self):
		ConfigListScreen.keyRight(self)
		self.updateConfigOptions()
		
	def updateConfigOptions(self):
		if self["config"].getCurrent()[1] == config.plugins.merlinhdmiin.adjustvolume:
			self.createList()
		
class MerlinHdmiInScreenSummary(Screen):
	skin = (
	"""	<screen name="MerlinHdmiInScreenSummary" position="0,0" size="132,64" id="1">
			<widget name="time" position="4,0" halign="center" valign="center" size="132,64" font="Regular;30"/>
		</screen>""",
	"""	<screen name="MerlinHdmiInScreenSummary" position="0,0" size="96,64" id="2">
			<widget name="time" position="0,0" halign="center" valign="center" size="96,64" font="Regular;30"/>
		</screen>""",
	"""	<screen name="MerlinHdmiInScreenSummary" position="0,0" size="400,240" id="3">
			<widget name="time" position="0,0" halign="center" valign="center" size="400,240" font="Regular;60"/>
		</screen>""",
	"""	<screen name="MerlinHdmiInScreenSummary" position="0,0" size="240,80" id="100">
			<widget name="time" position="0,0" halign="center" valign="center" size="240,80" font="Regular;40"/>
		</screen>"""
		 )
	
	def __init__(self,session,parent):
		Screen.__init__(self, session)
		self["time"] = Label(_("%s"))

		self.clockTimer = eTimer()
		self.clockTimer_conn = self.clockTimer.timeout.connect(self.updateTime)
		self.updateTime()
		
	def updateTime(self):
		now = t_strftime("%H:%M:%S",t_localtime())
		
		self["time"].setText(_("%s" %(now)))
		self.clockTimer.startLongTimer(1)	